# augmenter.py
import os

import audeer
import audiofile
import numpy as np
import pandas as pd
from audformat.utils import map_file_path
from audiomentations import (
    AddGaussianNoise,
    AddGaussianSNR,
    Compose,
    PitchShift,
    Shift,
    TimeStretch,
)
from nkululeko.util import Util
from tqdm import tqdm


class Augmenter:
    """
    augmenting the train split
    """

    def __init__(self, df):
        self.df = df
        self.util = Util("augmenter")
        # Define a standard transformation that randomly add augmentations to files
        self.audioment = Compose(
            [
                AddGaussianNoise(min_amplitude=0.001, max_amplitude=0.015, p=0.5),
                # AddGaussianSNR(min_snr_db=10, max_snr_db=40, p=0.5),
                TimeStretch(min_rate=0.8, max_rate=1.25, p=0.5),
                PitchShift(min_semitones=-4, max_semitones=4, p=0.5),
                Shift(min_fraction=-0.5, max_fraction=0.5, p=0.5),
            ]
        )

    def changepath(self, fp, np):
        #        parent = os.path.dirname(fp).split('/')[-1]
        fullpath = os.path.dirname(fp)
        #       newpath = f'{np}{parent}'
        #       audeer.mkdir(newpath)
        return fp.replace(fullpath, np)

    def augment(self, sample_selection):
        """
        augment the training files and return a dataframe with new files index.
        """
        files = self.df.index.get_level_values(0).values
        store = self.util.get_path("store")
        filepath = f"{store}augmentations/"
        audeer.mkdir(filepath)
        self.util.debug(f"augmenting {sample_selection} samples to {filepath}")
        newpath = ""
        index_map = {}
        for i, f in enumerate(tqdm(files)):
            signal, sr = audiofile.read(f)
            filename = os.path.basename(f)
            parent = os.path.dirname(f).split("/")[-1]
            sig_aug = self.audioment(samples=signal, sample_rate=sr)
            newpath = f"{filepath}/{parent}/"
            audeer.mkdir(newpath)
            new_full_name = newpath + filename
            audiofile.write(new_full_name, signal=sig_aug, sampling_rate=sr)
            index_map[f] = new_full_name
        df_ret = self.df.copy()
        file_index = df_ret.index.levels[0].map(lambda x: index_map[x]).values
        df_ret = df_ret.set_index(df_ret.index.set_levels(file_index, level="file"))

        return df_ret



#!/home/audeering.local/fburkhardt/research/nkululeko/venv/bin/python
# encoding: utf-8

# The MIT License

# Copyright (c) 2018 Ina (David Doukhan - http://www.ina.fr/)

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


import Pyro4
import sys
import os
import socket

from inaSpeechSegmenter import Segmenter


if __name__ == '__main__':
    dname = os.path.dirname(os.path.realpath(__file__))

    hostname = socket.gethostname()

    uri = sys.argv[1]
    jobserver = Pyro4.Proxy(uri)

    ret = -1
    outname = 'init'
    
    # batch size set at 1024. Use lower values with small gpus
    g = Segmenter(batch_size=1024)
    
    while True:
        lsrc, ldst = jobserver.get_njobs('%s %s' % (hostname, ret))
            
        print(lsrc, ldst)
        if len(lsrc) == 0:
            print('job list finished')
            break
        
        ret =  g.batch_process(lsrc, ldst, skipifexist=True, nbtry=3)



# augment.py
# augment the training sets

import argparse
import pandas as pd
import os
import ast
from nkululeko.experiment import Experiment
import configparser
from nkululeko.util import Util
from nkululeko.constants import VERSION


def main(src_dir):
    parser = argparse.ArgumentParser(description="Call the nkululeko framework.")
    parser.add_argument("--config", default="exp.ini", help="The base configuration")
    args = parser.parse_args()
    if args.config is not None:
        config_file = args.config
    else:
        config_file = f"{src_dir}/exp.ini"

    # test if the configuration file exists
    if not os.path.isfile(config_file):
        print(f"ERROR: no such file: {config_file}")
        exit()

    # load one configuration per experiment
    config = configparser.ConfigParser()
    config.read(config_file)
    # create a new experiment
    expr = Experiment(config)
    util = Util("augment")
    util.debug(
        f"running {expr.name} from config {config_file}, nkululeko version"
        f" {VERSION}"
    )

    if util.config_val("EXP", "no_warnings", False):
        import warnings

        warnings.filterwarnings("ignore")

    # load the data
    expr.load_datasets()

    # split into train and test
    expr.fill_train_and_tests()
    util.debug(f"train shape : {expr.df_train.shape}, test shape:{expr.df_test.shape}")

    # augment
    augmentings = util.config_val("AUGMENT", "augment", False)
    got_one = False
    if augmentings:
        augmentings = ast.literal_eval(augmentings)
        def_ret = pd.DataFrame()
        if "traditional" in augmentings:
            df_ret = expr.augment()
            got_one = True
        if "random_splice" in augmentings:
            df_ret = pd.concat([def_ret, expr.random_splice()])
            got_one = True
    if not augmentings:
        util.error("no augmentation selected")
    if not got_one:
        util.error(f"invalid augmentation(s): {augmentings}")
    # remove encoded labels
    target = util.config_val("DATA", "target", "emotion")
    if "class_label" in df_ret.columns:
        df_ret = df_ret.drop(columns=[target])
        df_ret = df_ret.rename(columns={"class_label": target})
    # save file
    filename = util.config_val("AUGMENT", "result", "augmented.csv")

    df_ret.to_csv(f"{expr.data_dir}/{filename}")
    util.debug(f"saved augmentation table to {filename} to {expr.data_dir}")
    print("DONE")


if __name__ == "__main__":
    cwd = os.path.dirname(os.path.abspath(__file__))
    main(cwd)  # use this if you want to state the config file path on command line
